
package Net::LDNS::RR::TALINK;

use parent 'Net::LDNS::RR';

1;

=head1 NAME

Net::LDNS::RR::TALINK - Type TALINK record

=head1 DESCRIPTION

A subclass of L<Net::LDNS::RR>, so it has all the methods of that class available in addition to the ones documented here.

=head1 METHODS

No RDATA methods implemented yet.

=cut
