package Net::LDNS::RR::A;

use parent 'Net::LDNS::RR';

1;

=head1 NAME

Net::LDNS::RR::A - Type A record

=head1 DESCRIPTION

A subclass of L<Net::LDNS::RR>, so it has all the methods of that class available in addition to the ones documented here.

=head1 METHODS

=over

=item address()

Returns the address.

=back
